package mx.controls
{
   import flash.display.DisplayObject;
   import flash.display.Graphics;
   import flash.events.Event;
   import flash.events.EventDispatcher;
   import flash.events.ProgressEvent;
   import flash.events.TimerEvent;
   import flash.system.ApplicationDomain;
   import flash.text.TextLineMetrics;
   import flash.utils.Timer;
   import mx.core.IFlexDisplayObject;
   import mx.core.UIComponent;
   import mx.core.UITextField;
   import mx.core.mx_internal;
   import mx.resources.ResourceBundle;
   import mx.styles.ISimpleStyleClient;
   
   use namespace mx_internal;
   
   public class ProgressBar extends UIComponent
   {
      private static var resourceLabel:String;
      
      mx_internal static const VERSION:String = "2.0.1.0";
      
      private static var packageResources:ResourceBundle = ResourceBundle.getResourceBundle("controls",ApplicationDomain.currentDomain);
      
      loadResources();
      
      private var _labelField:UITextField;
      
      private var trackSkinChanged:Boolean = false;
      
      private var _track:IFlexDisplayObject;
      
      private var barSkinChanged:Boolean = false;
      
      private var _labelPlacement:String = "bottom";
      
      private var _value:Number = 0;
      
      private var _maximum:Number = 0;
      
      private var stopPolledMode:Boolean = false;
      
      private var _source:Object;
      
      private var _barMask:UIComponent;
      
      private var _stringSource:String;
      
      private var indeterminateChanged:Boolean = true;
      
      private var stringSourceChanged:Boolean = false;
      
      private var _label:String;
      
      private var _direction:String = "right";
      
      private var _interval:Number = 30;
      
      private var sourceChanged:Boolean = false;
      
      private var _conversion:Number = 1;
      
      private var _mode:String = "event";
      
      private var pollTimer:Timer;
      
      private var _minimum:Number = 0;
      
      private var indeterminatePlaying:Boolean = false;
      
      private var indeterminateSkinChanged:Boolean = false;
      
      private var _determinateBar:IFlexDisplayObject;
      
      private var _indeterminate:Boolean = false;
      
      private var _content:UIComponent;
      
      private var _bar:UIComponent;
      
      private var modeChanged:Boolean = false;
      
      private var _indeterminateBar:IFlexDisplayObject;
      
      public function ProgressBar()
      {
         _interval = 30;
         indeterminatePlaying = false;
         stopPolledMode = false;
         barSkinChanged = false;
         trackSkinChanged = false;
         indeterminateSkinChanged = false;
         _conversion = 1;
         _direction = ProgressBarDirection.RIGHT;
         _indeterminate = false;
         indeterminateChanged = true;
         _labelPlacement = ProgressBarLabelPlacement.BOTTOM;
         _maximum = 0;
         _minimum = 0;
         _mode = ProgressBarMode.EVENT;
         modeChanged = false;
         sourceChanged = false;
         stringSourceChanged = false;
         _value = 0;
         super();
         bundleChanged();
         pollTimer = new Timer(_interval);
      }
      
      private static function loadResources() : void
      {
         resourceLabel = packageResources.getString("label");
      }
      
      [Bindable("minimumChanged")]
      public function get minimum() : Number
      {
         return _minimum;
      }
      
      [Bindable("sourceChanged")]
      public function get source() : Object
      {
         return _source;
      }
      
      public function set minimum(param1:Number) : void
      {
         if(!isNaN(param1) && _mode == ProgressBarMode.MANUAL && param1 != _minimum)
         {
            _minimum = param1;
            invalidateDisplayList();
            dispatchEvent(new Event("minimumChanged"));
         }
      }
      
      private function completeHandler(param1:Event) : void
      {
         dispatchEvent(param1);
         invalidateDisplayList();
      }
      
      public function set maximum(param1:Number) : void
      {
         if(!isNaN(param1) && _mode == ProgressBarMode.MANUAL && param1 != _maximum)
         {
            _maximum = param1;
            invalidateDisplayList();
            dispatchEvent(new Event("maximumChanged"));
         }
      }
      
      private function updateIndeterminateHandler(param1:Event) : void
      {
         if(_indeterminateBar.x < 1)
         {
            _indeterminateBar.x += 1;
         }
         else
         {
            _indeterminateBar.x = -26;
         }
      }
      
      [Bindable("progress")]
      public function get percentComplete() : Number
      {
         var _loc1_:Number = NaN;
         if(_value < _minimum || _maximum < _minimum)
         {
            return 0;
         }
         if(_maximum - _minimum == 0)
         {
            return 0;
         }
         _loc1_ = 100 * (_value - _minimum) / (_maximum - _minimum);
         if(isNaN(_loc1_) || _loc1_ < 0)
         {
            return 0;
         }
         if(_loc1_ > 100)
         {
            return 100;
         }
         return _loc1_;
      }
      
      [Bindable("indeterminateChanged")]
      public function get indeterminate() : Boolean
      {
         return _indeterminate;
      }
      
      private function getFullLabelText() : String
      {
         var _loc1_:Number = NaN;
         var _loc2_:Number = NaN;
         var _loc3_:String = null;
         _loc1_ = Math.max(_value,0);
         _loc2_ = Math.max(_maximum,0);
         _loc3_ = label;
         if(label)
         {
            if(_indeterminate)
            {
               _loc3_ = _loc3_.replace("%1",String(Math.floor(_loc1_ / _conversion)));
               _loc3_ = _loc3_.replace("%2","??");
               _loc3_ = _loc3_.replace("%3","");
               _loc3_ = _loc3_.replace("%%","");
            }
            else
            {
               _loc3_ = _loc3_.replace("%1",String(Math.floor(_loc1_ / _conversion)));
               _loc3_ = _loc3_.replace("%2",String(Math.floor(_loc2_ / _conversion)));
               _loc3_ = _loc3_.replace("%3",String(Math.floor(percentComplete)));
               _loc3_ = _loc3_.replace("%%","%");
            }
         }
         return _loc3_;
      }
      
      public function setProgress(param1:Number, param2:Number) : void
      {
         if(_mode == ProgressBarMode.MANUAL)
         {
            _setProgress(param1,param2);
         }
      }
      
      public function set source(param1:Object) : void
      {
         var value:Object = param1;
         if(value is String)
         {
            _stringSource = String(value);
            try
            {
               value = document[_stringSource];
            }
            catch(e:Error)
            {
               stringSourceChanged = true;
            }
         }
         if(value)
         {
            _source = value;
            sourceChanged = true;
            modeChanged = true;
            indeterminateChanged = true;
            invalidateProperties();
         }
         else if(_source != null)
         {
            pollTimer.reset();
         }
      }
      
      public function set labelPlacement(param1:String) : void
      {
         if(param1 != _labelPlacement)
         {
            _labelPlacement = param1;
         }
         invalidateSize();
         invalidateDisplayList();
         dispatchEvent(new Event("labelPlacementChanged"));
      }
      
      private function progressHandler(param1:ProgressEvent) : void
      {
         _setProgress(param1.bytesLoaded,param1.bytesTotal);
      }
      
      [Bindable("conversionChanged")]
      public function get conversion() : Number
      {
         return _conversion;
      }
      
      override protected function measure() : void
      {
         var _loc1_:Number = NaN;
         var _loc2_:Number = NaN;
         var _loc3_:Number = NaN;
         var _loc4_:Number = NaN;
         var _loc5_:Number = NaN;
         var _loc6_:Number = NaN;
         var _loc7_:Number = NaN;
         var _loc8_:Number = NaN;
         var _loc9_:Number = NaN;
         var _loc10_:Number = NaN;
         var _loc11_:Number = NaN;
         var _loc12_:Number = NaN;
         var _loc13_:Number = NaN;
         var _loc14_:Number = NaN;
         var _loc15_:TextLineMetrics = null;
         var _loc16_:Number = NaN;
         var _loc17_:Number = NaN;
         super.measure();
         _loc3_ = NaN;
         _loc4_ = NaN;
         _loc5_ = getStyle("trackHeight");
         _loc6_ = Number(_track.measuredWidth);
         _loc7_ = isNaN(_loc5_) ? Number(_track.measuredHeight) : _loc5_;
         _loc8_ = getStyle("horizontalGap");
         _loc9_ = getStyle("verticalGap");
         _loc10_ = getStyle("paddingLeft");
         _loc11_ = getStyle("paddingRight");
         _loc12_ = getStyle("paddingTop");
         _loc13_ = getStyle("paddingBottom");
         _loc14_ = getStyle("labelWidth");
         _loc15_ = measureText(predictLabelText());
         _loc16_ = isNaN(_loc14_) ? _loc15_.width + UITextField.mx_internal::TEXT_WIDTH_PADDING : _loc14_;
         _loc17_ = _loc15_.height + UITextField.mx_internal::TEXT_HEIGHT_PADDING;
         switch(labelPlacement)
         {
            case ProgressBarLabelPlacement.LEFT:
            case ProgressBarLabelPlacement.RIGHT:
               _loc1_ = _loc16_ + _loc6_ + _loc10_ + _loc11_ + _loc8_;
               _loc2_ = Math.max(_loc17_,_loc7_) + _loc12_ + _loc13_;
               measuredMinWidth = _loc1_;
               break;
            case ProgressBarLabelPlacement.CENTER:
               _loc1_ = Math.max(_loc16_,_loc6_) + _loc10_ + _loc11_ + _loc8_;
               _loc2_ = Math.max(_loc17_,_loc7_) + _loc12_ + _loc13_;
               measuredMinWidth = _loc16_;
               break;
            default:
               _loc1_ = Math.max(_loc16_,_loc6_) + _loc10_ + _loc11_;
               _loc2_ = _loc17_ + _loc7_ + _loc12_ + _loc13_ + _loc9_;
               measuredMinWidth = _loc16_;
         }
         measuredWidth = _loc1_;
         measuredMinHeight = measuredHeight = _loc2_;
         if(!isNaN(_loc3_))
         {
            measuredMinWidth = _loc3_;
         }
         if(!isNaN(_loc4_))
         {
            measuredMinHeight = _loc4_;
         }
      }
      
      public function set indeterminate(param1:Boolean) : void
      {
         _indeterminate = param1;
         indeterminateChanged = true;
         invalidateProperties();
         invalidateDisplayList();
         dispatchEvent(new Event("indeterminateChanged"));
      }
      
      private function createBar() : void
      {
         var _loc1_:Class = null;
         if(_determinateBar)
         {
            _bar.removeChild(DisplayObject(_determinateBar));
            _determinateBar = null;
         }
         _loc1_ = getStyle("barSkin");
         if(_loc1_)
         {
            _determinateBar = new _loc1_();
            if(_determinateBar is ISimpleStyleClient)
            {
               ISimpleStyleClient(_determinateBar).styleName = this;
            }
            _bar.addChild(DisplayObject(_determinateBar));
         }
      }
      
      private function predictLabelText() : String
      {
         var _loc1_:String = null;
         var _loc2_:Number = NaN;
         var _loc3_:String = null;
         _loc1_ = label;
         if(_maximum != 0)
         {
            _loc2_ = _maximum;
         }
         else
         {
            _loc2_ = 100000;
         }
         if(label)
         {
            if(_indeterminate)
            {
               _loc1_ = _loc1_.replace("%1",String(Math.floor(_loc2_ / _conversion)));
               _loc1_ = _loc1_.replace("%2","??");
               _loc1_ = _loc1_.replace("%3","");
               _loc1_ = _loc1_.replace("%%","");
            }
            else
            {
               _loc1_ = _loc1_.replace("%1",String(Math.floor(_loc2_ / _conversion)));
               _loc1_ = _loc1_.replace("%2",String(Math.floor(_loc2_ / _conversion)));
               _loc1_ = _loc1_.replace("%3","100");
               _loc1_ = _loc1_.replace("%%","%");
            }
         }
         _loc3_ = getFullLabelText();
         if(_loc1_.length > _loc3_.length)
         {
            return _loc1_;
         }
         return _loc3_;
      }
      
      [Bindable("modeChanged")]
      public function get mode() : String
      {
         return _mode;
      }
      
      [Bindable("change")]
      public function get value() : Number
      {
         return _value;
      }
      
      private function createTrack() : void
      {
         var _loc1_:Class = null;
         if(_track)
         {
            _content.removeChild(DisplayObject(_track));
            _track = null;
         }
         _loc1_ = getStyle("trackSkin");
         if(_loc1_)
         {
            _track = new _loc1_();
            if(_track is ISimpleStyleClient)
            {
               ISimpleStyleClient(_track).styleName = this;
            }
            _content.addChildAt(DisplayObject(_track),0);
         }
      }
      
      override protected function commitProperties() : void
      {
         super.commitProperties();
         if(trackSkinChanged)
         {
            trackSkinChanged = false;
            createTrack();
         }
         if(barSkinChanged)
         {
            barSkinChanged = false;
            createBar();
         }
         if(indeterminateSkinChanged)
         {
            indeterminateSkinChanged = false;
            createIndeterminateBar();
         }
         if(stringSourceChanged)
         {
            stringSourceChanged = false;
            try
            {
               _source = document[_stringSource];
            }
            catch(e:Error)
            {
            }
         }
         if(sourceChanged)
         {
            sourceChanged = false;
            dispatchEvent(new Event("sourceChanged"));
         }
         if(modeChanged)
         {
            modeChanged = false;
            if(_source)
            {
               if(_mode == ProgressBarMode.EVENT)
               {
                  if(_source is EventDispatcher)
                  {
                     _source.addEventListener(ProgressEvent.PROGRESS,progressHandler);
                     _source.addEventListener(Event.COMPLETE,completeHandler);
                  }
                  else
                  {
                     _source = null;
                  }
               }
               else
               {
                  _source.removeEventListener(ProgressEvent.PROGRESS,progressHandler);
                  _source.removeEventListener(Event.COMPLETE,completeHandler);
               }
            }
            if(_mode == ProgressBarMode.POLLED)
            {
               pollTimer.addEventListener(TimerEvent.TIMER,updatePolledHandler);
               pollTimer.start();
            }
            else if(stopPolledMode)
            {
               stopPolledMode = false;
               pollTimer.removeEventListener(TimerEvent.TIMER,updatePolledHandler);
               pollTimer.reset();
            }
            dispatchEvent(new Event("modeChanged"));
         }
      }
      
      public function set label(param1:String) : void
      {
         _label = param1;
         invalidateDisplayList();
         dispatchEvent(new Event("labelChanged"));
      }
      
      private function layoutContent(param1:Number, param2:Number) : void
      {
         _track.move(0,0);
         _track.setActualSize(param1,param2);
         _bar.move(0,0);
         _determinateBar.move(0,0);
         _indeterminateBar.setActualSize(param1 + 28,param2);
      }
      
      [Bindable("maximumChanged")]
      public function get maximum() : Number
      {
         return _maximum;
      }
      
      public function set conversion(param1:Number) : void
      {
         if(!isNaN(param1) && Number(param1) > 0 && param1 != _conversion)
         {
            _conversion = Number(param1);
            invalidateDisplayList();
            dispatchEvent(new Event("conversionChanged"));
         }
      }
      
      private function stopPlayingIndeterminate() : void
      {
         if(indeterminatePlaying)
         {
            indeterminatePlaying = false;
            pollTimer.removeEventListener(TimerEvent.TIMER,updateIndeterminateHandler);
            if(_mode != ProgressBarMode.POLLED)
            {
               pollTimer.reset();
            }
         }
      }
      
      [Bindable("labelPlacementChanged")]
      public function get labelPlacement() : String
      {
         return _labelPlacement;
      }
      
      private function createIndeterminateBar() : void
      {
         var _loc1_:Class = null;
         if(_indeterminateBar)
         {
            _bar.removeChild(DisplayObject(_indeterminateBar));
            _indeterminateBar = null;
         }
         _loc1_ = getStyle("indeterminateSkin");
         if(_loc1_)
         {
            _indeterminateBar = new _loc1_();
            if(_indeterminateBar is ISimpleStyleClient)
            {
               ISimpleStyleClient(_indeterminateBar).styleName = this;
            }
            _indeterminateBar.visible = false;
            _bar.addChild(DisplayObject(_indeterminateBar));
         }
      }
      
      private function updatePolledHandler(param1:Event) : void
      {
         var _loc2_:Object = null;
         var _loc3_:Number = NaN;
         var _loc4_:Number = NaN;
         if(_source)
         {
            _loc2_ = _source;
            _loc3_ = Number(_loc2_.bytesLoaded);
            _loc4_ = Number(_loc2_.bytesTotal);
            if(!isNaN(_loc3_) && !isNaN(_loc4_))
            {
               _setProgress(_loc3_,_loc4_);
               if(percentComplete >= 100 && _value > 0)
               {
                  pollTimer.reset();
               }
            }
         }
      }
      
      private function startPlayingIndeterminate() : void
      {
         if(!indeterminatePlaying)
         {
            indeterminatePlaying = true;
            pollTimer.addEventListener(TimerEvent.TIMER,updateIndeterminateHandler);
            pollTimer.start();
         }
      }
      
      [Bindable("labelChanged")]
      public function get label() : String
      {
         return _label;
      }
      
      public function set direction(param1:String) : void
      {
         if(param1 == ProgressBarDirection.LEFT || param1 == ProgressBarDirection.RIGHT)
         {
            _direction = param1;
         }
         invalidateDisplayList();
         dispatchEvent(new Event("directionChanged"));
      }
      
      private function _setProgress(param1:Number, param2:Number) : void
      {
         var _loc3_:ProgressEvent = null;
         if(enabled && !isNaN(param1) && !isNaN(param2))
         {
            _value = param1;
            _maximum = param2;
            dispatchEvent(new Event(Event.CHANGE));
            _loc3_ = new ProgressEvent(ProgressEvent.PROGRESS);
            _loc3_.bytesLoaded = param1;
            _loc3_.bytesTotal = param2;
            dispatchEvent(_loc3_);
            if(_indeterminate)
            {
               startPlayingIndeterminate();
            }
            if(_value == _maximum && _value > 0)
            {
               if(_indeterminate)
               {
                  stopPlayingIndeterminate();
               }
               if(mode != ProgressBarMode.EVENT)
               {
                  dispatchEvent(new Event(Event.COMPLETE));
               }
            }
            invalidateDisplayList();
         }
      }
      
      override protected function createChildren() : void
      {
         super.createChildren();
         if(!_content)
         {
            _content = new UIComponent();
            addChild(_content);
         }
         if(!_bar)
         {
            _bar = new UIComponent();
            _content.addChild(_bar);
         }
         if(!_barMask)
         {
            _barMask = new UIComponent();
            _barMask.visible = true;
            _bar.addChild(_barMask);
            UIComponent(_bar).mask = _barMask;
         }
         if(!_labelField)
         {
            _labelField = new UITextField();
            _labelField.styleName = this;
            addChild(_labelField);
         }
      }
      
      public function set mode(param1:String) : void
      {
         if(param1 != _mode)
         {
            if(_mode == ProgressBarMode.POLLED)
            {
               stopPolledMode = true;
            }
            _mode = param1;
            modeChanged = true;
            indeterminateChanged = true;
            invalidateProperties();
            invalidateDisplayList();
         }
      }
      
      [Bindable("directionChanged")]
      public function get direction() : String
      {
         return _direction;
      }
      
      override public function styleChanged(param1:String) : void
      {
         var _loc2_:Boolean = false;
         _loc2_ = false;
         super.styleChanged(param1);
         if(param1 == null || param1 == "styleName")
         {
            barSkinChanged = trackSkinChanged = indeterminateSkinChanged = true;
            _loc2_ = true;
         }
         else if(param1 == "barSkin")
         {
            barSkinChanged = true;
            _loc2_ = true;
         }
         else if(param1 == "trackSkin")
         {
            trackSkinChanged = true;
            _loc2_ = true;
         }
         else if(param1 == "indeterminateSkin")
         {
            indeterminateSkinChanged = true;
            _loc2_ = true;
         }
         if(_loc2_)
         {
            invalidateProperties();
            invalidateSize();
            invalidateDisplayList();
         }
      }
      
      private function bundleChanged() : void
      {
         _label = resourceLabel;
      }
      
      override protected function childrenCreated() : void
      {
         super.childrenCreated();
         trackSkinChanged = true;
         barSkinChanged = true;
         indeterminateSkinChanged = true;
      }
      
      override protected function updateDisplayList(param1:Number, param2:Number) : void
      {
         var _loc3_:Number = NaN;
         var _loc4_:Number = NaN;
         var _loc5_:Number = NaN;
         var _loc6_:Number = NaN;
         var _loc7_:Number = NaN;
         var _loc8_:Number = NaN;
         var _loc9_:Number = NaN;
         var _loc10_:Number = NaN;
         var _loc11_:Number = NaN;
         var _loc12_:Number = NaN;
         var _loc13_:TextLineMetrics = null;
         var _loc14_:Number = NaN;
         var _loc15_:Number = NaN;
         var _loc16_:Graphics = null;
         var _loc17_:Number = NaN;
         var _loc18_:Number = NaN;
         super.updateDisplayList(param1,param2);
         _loc3_ = getStyle("horizontalGap");
         _loc4_ = getStyle("verticalGap");
         _loc5_ = getStyle("paddingLeft");
         _loc6_ = getStyle("paddingRight");
         _loc7_ = getStyle("paddingTop");
         _loc8_ = getStyle("paddingBottom");
         _loc9_ = _loc5_;
         _loc10_ = _loc7_;
         _loc11_ = getStyle("labelWidth");
         _loc12_ = getStyle("trackHeight");
         _loc12_ = isNaN(_loc12_) ? Number(_track.measuredHeight) : _loc12_;
         _loc13_ = measureText(predictLabelText());
         _loc14_ = isNaN(_loc11_) ? _loc13_.width + UITextField.mx_internal::TEXT_WIDTH_PADDING : _loc11_;
         _loc15_ = _loc13_.height + UITextField.mx_internal::TEXT_HEIGHT_PADDING;
         switch(labelPlacement)
         {
            case ProgressBarLabelPlacement.TOP:
               _labelField.move(_loc9_,_loc10_);
               _labelField.setActualSize(_loc14_,_loc15_);
               _content.move(_loc9_,_loc10_ + _loc15_ + _loc4_);
               layoutContent(param1 - _loc9_ - _loc6_,_loc12_);
               break;
            case ProgressBarLabelPlacement.RIGHT:
               _loc17_ = param1 - _loc9_ - _loc6_ - _loc14_ - _loc3_;
               _labelField.move(_loc9_ + _loc17_ + _loc3_,(param2 - _loc15_) / 2);
               _labelField.setActualSize(_loc14_,_loc15_);
               _content.move(_loc9_,_loc10_ + (_loc15_ - _loc12_) / 2);
               layoutContent(_loc17_,_loc12_);
               break;
            case ProgressBarLabelPlacement.LEFT:
               _labelField.move(_loc9_,_loc10_ + (param2 - _loc15_) / 2);
               _labelField.setActualSize(_loc14_,_loc15_);
               _content.move(_loc9_ + _loc14_ + _loc3_,_loc10_ + (_loc15_ - _loc12_) / 2);
               layoutContent(param1 - _loc9_ - _loc14_ - _loc4_ - _loc6_,_loc12_);
               break;
            case ProgressBarLabelPlacement.CENTER:
               _labelField.move((param1 - _loc14_) / 2,(param2 - _loc15_) / 2);
               _labelField.setActualSize(_loc14_,_loc15_);
               _content.move(_loc9_,_loc10_);
               layoutContent(param1 - _loc6_,param2 - _loc8_);
               break;
            default:
               _labelField.move(_loc9_,_loc10_ + _loc12_ + _loc4_);
               _labelField.setActualSize(_loc14_,_loc15_);
               _content.move(_loc9_,_loc10_);
               layoutContent(param1 - _loc9_ - _loc6_,_loc12_);
         }
         _barMask.move(0,0);
         _loc16_ = _barMask.graphics;
         _loc16_.clear();
         _loc16_.beginFill(16776960);
         _loc16_.drawRect(1,1,_track.width - 2,_track.height - 2);
         _loc16_.endFill();
         _labelField.text = getFullLabelText();
         _indeterminateBar.visible = _indeterminate;
         if(indeterminateChanged)
         {
            indeterminateChanged = false;
            _indeterminateBar.visible = _indeterminate;
            if(_indeterminate && _source == null && _mode == ProgressBarMode.EVENT)
            {
               startPlayingIndeterminate();
            }
            else
            {
               stopPlayingIndeterminate();
            }
         }
         if(_indeterminate)
         {
            _determinateBar.setActualSize(_track.width,_track.height);
         }
         else
         {
            _loc18_ = Math.max(0,_track.width * percentComplete / 100);
            _determinateBar.setActualSize(_loc18_,_track.height);
            _determinateBar.x = direction == ProgressBarDirection.RIGHT ? 0 : _track.width - _loc18_;
         }
      }
   }
}

