package mx.messaging
{
   import flash.errors.IllegalOperationError;
   import flash.events.EventDispatcher;
   import flash.events.TimerEvent;
   import flash.utils.Timer;
   import mx.core.mx_internal;
   import mx.events.PropertyChangeEvent;
   import mx.messaging.channels.NetConnectionChannel;
   import mx.messaging.config.ServerConfig;
   import mx.messaging.errors.NoChannelAvailableError;
   import mx.messaging.events.ChannelEvent;
   import mx.messaging.events.ChannelFaultEvent;
   import mx.messaging.events.MessageEvent;
   import mx.messaging.messages.CommandMessage;
   import mx.messaging.messages.ErrorMessage;
   import mx.messaging.messages.IMessage;
   import mx.utils.Translator;
   
   use namespace mx_internal;
   
   public class ChannelSet extends EventDispatcher
   {
      private static const TRANSLATOR:Translator = Translator.getDefaultInstanceFor(ChannelSet);
      
      private var _shouldHunt:Boolean;
      
      private var _connected:Boolean;
      
      private var _hasRequestedClusterEndpoints:Boolean;
      
      private var _clustered:Boolean;
      
      private var _pendingMessageIds:Object;
      
      private var _channels:Array;
      
      private var _hunting:Boolean;
      
      private var _credentials:String;
      
      private var _connecting:Boolean;
      
      private var _shouldBeConnected:Boolean;
      
      private var _configured:Boolean;
      
      private var _channelIds:Array;
      
      private var _currentChannel:Channel;
      
      private var _currentChannelIndex:int;
      
      private var _pendingSends:Array;
      
      private var _messageAgents:Array;
      
      private var _channelFailoverURIs:Object;
      
      public function ChannelSet(param1:Array = null, param2:Boolean = false)
      {
         super();
         _clustered = param2;
         _connected = false;
         _connecting = false;
         _currentChannelIndex = -1;
         if(param1 != null)
         {
            _channelIds = param1;
            _channels = new Array(_channelIds.length);
            _configured = true;
         }
         else
         {
            _channels = [];
            _configured = false;
         }
         _hasRequestedClusterEndpoints = false;
         _hunting = false;
         _messageAgents = [];
         _pendingMessageIds = {};
         _pendingSends = [];
         _shouldBeConnected = false;
         _shouldHunt = true;
      }
      
      public function send(param1:MessageAgent, param2:IMessage) : void
      {
         var _loc3_:Timer = null;
         if(connected)
         {
            _currentChannel.send(param1,param2);
         }
         else
         {
            if(_pendingMessageIds[param2.messageId] == null)
            {
               _pendingMessageIds[param2.messageId] = true;
               _pendingSends.push(new PendingSend(param1,param2));
            }
            if(!_connecting)
            {
               if(_currentChannel == null || _currentChannelIndex == -1)
               {
                  hunt();
               }
               if(_currentChannel is NetConnectionChannel)
               {
                  _loc3_ = new Timer(1,1);
                  _loc3_.addEventListener(TimerEvent.TIMER,reconnectChannel);
                  _loc3_.start();
               }
               else
               {
                  connectChannel();
               }
            }
         }
      }
      
      [Bindable(event="propertyChange")]
      public function get connected() : Boolean
      {
         return _connected;
      }
      
      private function hunt() : Boolean
      {
         if(_channels.length == 0)
         {
            throw new NoChannelAvailableError(TRANSLATOR.textOf("noAvailableChannels"));
         }
         if(++_currentChannelIndex >= _channels.length)
         {
            _currentChannelIndex = -1;
            return false;
         }
         if(_currentChannelIndex > 0)
         {
            _hunting = true;
         }
         if(configured)
         {
            if(_channels[_currentChannelIndex] != null)
            {
               _currentChannel = _channels[_currentChannelIndex];
            }
            else
            {
               _currentChannel = ServerConfig.getChannel(_channelIds[_currentChannelIndex],_clustered);
               _currentChannel.setCredentials(_credentials);
               _channels[_currentChannelIndex] = _currentChannel;
            }
         }
         else
         {
            _currentChannel = _channels[_currentChannelIndex];
         }
         if(_channelFailoverURIs != null && _channelFailoverURIs[_currentChannel.id] != null)
         {
            _currentChannel.failoverURIs = _channelFailoverURIs[_currentChannel.id];
         }
         return true;
      }
      
      public function get clustered() : Boolean
      {
         return _clustered;
      }
      
      public function disconnect(param1:MessageAgent) : void
      {
         var _loc2_:int = 0;
         var _loc3_:int = 0;
         var _loc4_:int = 0;
         var _loc5_:PendingSend = null;
         _loc2_ = int(_messageAgents.length);
         _loc3_ = 0;
         while(_loc3_ < _loc2_)
         {
            if(_messageAgents[_loc3_] == param1)
            {
               _messageAgents.splice(_loc3_,1);
               removeEventListener(ChannelEvent.CONNECT,param1.channelConnectHandler);
               removeEventListener(ChannelEvent.DISCONNECT,param1.channelDisconnectHandler);
               removeEventListener(ChannelFaultEvent.FAULT,param1.channelFaultHandler);
               if(connected || _connecting)
               {
                  param1.channelDisconnectHandler(ChannelEvent.createEvent(ChannelEvent.DISCONNECT,_currentChannel,false));
               }
               else
               {
                  _loc4_ = int(_pendingSends.length);
                  _loc3_ = 0;
                  while(_loc3_ < _loc4_)
                  {
                     _loc5_ = PendingSend(_pendingSends[_loc3_]);
                     if(_loc5_.agent == param1)
                     {
                        _pendingSends.splice(_loc3_,1);
                        _loc3_--;
                        _loc4_--;
                        delete _pendingMessageIds[_loc5_.message.messageId];
                     }
                     _loc3_++;
                  }
               }
               break;
            }
            _loc3_++;
         }
         if(_messageAgents.length == 0)
         {
            _shouldBeConnected = false;
            if(connected)
            {
               disconnectChannel();
            }
         }
      }
      
      public function addChannel(param1:Channel) : void
      {
         var _loc2_:int = 0;
         var _loc3_:int = 0;
         if(configured)
         {
            throw new IllegalOperationError(TRANSLATOR.textOf("cannotAddWhenConfigured"));
         }
         _loc2_ = int(_channels.length);
         _loc3_ = 0;
         while(_loc3_ < _loc2_)
         {
            if(_channels[_loc3_] == param1)
            {
               return;
            }
            _loc3_++;
         }
         _channels.push(param1);
         if(_credentials)
         {
            param1.setCredentials(_credentials);
         }
      }
      
      public function logout(param1:MessageAgent) : void
      {
         var _loc2_:int = 0;
         var _loc3_:int = 0;
         _credentials = null;
         _loc2_ = int(_channels.length);
         _loc3_ = 0;
         while(_loc3_ < _loc2_)
         {
            if(_channels[_loc3_] != null)
            {
               _channels[_loc3_].logout(param1);
            }
            _loc3_++;
         }
      }
      
      public function get channelIds() : Array
      {
         var _loc1_:Array = null;
         var _loc2_:int = 0;
         var _loc3_:int = 0;
         if(_channelIds != null)
         {
            return _channelIds;
         }
         _loc1_ = [];
         _loc2_ = int(_channels.length);
         _loc3_ = 0;
         while(_loc3_ < _loc2_)
         {
            _loc1_.push(_channels[_loc3_].id);
            _loc3_++;
         }
         return _loc1_;
      }
      
      private function connectChannel() : void
      {
         if(!connected && !_connecting)
         {
            _connecting = true;
            _currentChannel.connect(this);
            _currentChannel.addEventListener(MessageEvent.MESSAGE,messageHandler);
         }
      }
      
      private function reconnectChannel(param1:TimerEvent) : void
      {
         var _loc2_:Timer = null;
         _loc2_ = Timer(param1.target);
         _loc2_.stop();
         _loc2_.removeEventListener(TimerEvent.TIMER,reconnectChannel);
         connectChannel();
      }
      
      public function get configured() : Boolean
      {
         return _configured;
      }
      
      public function setCredentials(param1:String, param2:MessageAgent) : void
      {
         var _loc3_:int = 0;
         var _loc4_:int = 0;
         _credentials = param1;
         _loc3_ = int(_channels.length);
         _loc4_ = 0;
         while(_loc4_ < _loc3_)
         {
            if(_channels[_loc4_] != null)
            {
               _channels[_loc4_].setCredentials(_credentials,param2);
            }
            _loc4_++;
         }
      }
      
      private function messageHandler(param1:MessageEvent) : void
      {
         dispatchEvent(MessageEvent.createEvent(param1.message.destination,param1.message));
      }
      
      public function get currentChannel() : Channel
      {
         return _currentChannel;
      }
      
      public function channelDisconnectHandler(param1:ChannelEvent) : void
      {
         var _loc2_:Timer = null;
         _connecting = false;
         setConnected(false);
         if(_shouldBeConnected && !param1.reconnecting && !param1.rejected)
         {
            if(_shouldHunt && hunt())
            {
               param1.reconnecting = true;
               dispatchEvent(param1);
               if(_currentChannel is NetConnectionChannel)
               {
                  _loc2_ = new Timer(1,1);
                  _loc2_.addEventListener(TimerEvent.TIMER,reconnectChannel);
                  _loc2_.start();
               }
               else
               {
                  connectChannel();
               }
            }
            else
            {
               _shouldHunt = true;
               param1.reconnecting = false;
               dispatchEvent(param1);
               faultPendingSends(param1);
            }
         }
         else
         {
            dispatchEvent(param1);
         }
      }
      
      protected function setConnected(param1:Boolean) : void
      {
         var _loc2_:PropertyChangeEvent = null;
         if(_connected != param1)
         {
            _loc2_ = PropertyChangeEvent.createUpdateEvent(this,"connected",_connected,param1);
            _connected = param1;
            dispatchEvent(_loc2_);
         }
      }
      
      private function disconnectChannel() : void
      {
         _connecting = false;
         _currentChannel.disconnect(this);
         _currentChannel.removeEventListener(MessageEvent.MESSAGE,messageHandler);
      }
      
      public function removeChannel(param1:Channel) : void
      {
         var _loc2_:int = 0;
         var _loc3_:int = 0;
         if(configured)
         {
            throw new IllegalOperationError(TRANSLATOR.textOf("cannotRemoveWhenConfigured"));
         }
         _loc2_ = int(_channels.length);
         _loc3_ = 0;
         while(_loc3_ < _loc2_)
         {
            if(_channels[_loc3_] == param1)
            {
               _channels.splice(_loc3_,1);
               if(_currentChannel != null && _currentChannel == param1)
               {
                  if(connected)
                  {
                     _shouldHunt = false;
                     disconnectChannel();
                  }
                  _currentChannel = null;
                  _currentChannelIndex = -1;
               }
               break;
            }
            _loc3_++;
         }
      }
      
      private function faultPendingSends(param1:ChannelEvent) : void
      {
         var _loc2_:PendingSend = null;
         var _loc3_:IMessage = null;
         var _loc4_:String = null;
         var _loc5_:ErrorMessage = null;
         var _loc6_:ChannelFaultEvent = null;
         while(_pendingSends.length > 0)
         {
            _loc2_ = _pendingSends.shift() as PendingSend;
            _loc3_ = _loc2_.message;
            _loc4_ = _loc3_.messageId;
            delete _pendingMessageIds[_loc4_];
            _loc5_ = new ErrorMessage();
            _loc5_.correlationId = _loc4_;
            _loc5_.headers[ErrorMessage.RETRYABLE_HINT_HEADER] = true;
            _loc5_.faultCode = "Client.Error.MessageSend";
            _loc5_.faultString = TRANSLATOR.textOf("sendFailed");
            if(param1 is ChannelFaultEvent)
            {
               _loc6_ = param1 as ChannelFaultEvent;
               _loc5_.faultDetail = _loc6_.faultCode + " " + _loc6_.faultString + " " + _loc6_.faultDetail;
               if(_loc6_.rootCause != null)
               {
                  _loc5_.rootCause = _loc6_.rootCause;
               }
            }
            else
            {
               _loc5_.faultDetail = TRANSLATOR.textOf("cannotConnectToDestination");
            }
            _loc2_.agent.fault(_loc5_,_loc3_);
         }
      }
      
      public function channelFaultHandler(param1:ChannelFaultEvent) : void
      {
         var _loc2_:Timer = null;
         if(param1.channel.connected)
         {
            dispatchEvent(param1);
         }
         else
         {
            _connecting = false;
            setConnected(false);
            if(_shouldBeConnected && !param1.reconnecting)
            {
               if(hunt())
               {
                  param1.reconnecting = true;
                  dispatchEvent(param1);
                  if(_currentChannel is NetConnectionChannel)
                  {
                     _loc2_ = new Timer(1,1);
                     _loc2_.addEventListener(TimerEvent.TIMER,reconnectChannel);
                     _loc2_.start();
                  }
                  else
                  {
                     connectChannel();
                  }
               }
               else
               {
                  param1.reconnecting = false;
                  dispatchEvent(param1);
                  faultPendingSends(param1);
               }
            }
            else
            {
               dispatchEvent(param1);
            }
         }
      }
      
      override public function toString() : String
      {
         var _loc1_:String = null;
         var _loc2_:uint = 0;
         _loc1_ = "[ChannelSet ";
         _loc2_ = 0;
         while(_loc2_ < _channels.length)
         {
            if(_channels[_loc2_] != null)
            {
               _loc1_ += _channels[_loc2_].id + " ";
            }
            _loc2_++;
         }
         return _loc1_ + "]";
      }
      
      public function connect(param1:MessageAgent) : void
      {
         var _loc2_:int = 0;
         var _loc3_:int = 0;
         _loc2_ = int(_messageAgents.length);
         _loc3_ = 0;
         while(_loc3_ < _loc2_)
         {
            if(_messageAgents[_loc3_] == param1)
            {
               return;
            }
            _loc3_++;
         }
         _shouldBeConnected = true;
         _messageAgents.push(param1);
         addEventListener(ChannelEvent.CONNECT,param1.channelConnectHandler);
         addEventListener(ChannelEvent.DISCONNECT,param1.channelDisconnectHandler);
         addEventListener(ChannelFaultEvent.FAULT,param1.channelFaultHandler);
         if(connected)
         {
            param1.channelConnectHandler(ChannelEvent.createEvent(ChannelEvent.CONNECT,_currentChannel,false));
         }
      }
      
      public function channelConnectHandler(param1:ChannelEvent) : void
      {
         var _loc2_:PendingSend = null;
         var _loc3_:Object = null;
         var _loc4_:CommandMessage = null;
         _connecting = false;
         setConnected(true);
         while(_pendingSends.length > 0)
         {
            _loc2_ = PendingSend(_pendingSends.shift());
            delete _pendingMessageIds[_loc2_.message.messageId];
            send(_loc2_.agent,_loc2_.message);
         }
         if(!_hasRequestedClusterEndpoints && clustered)
         {
            _loc3_ = ServerConfig.mx_internal::getChannelSetClusterRequestParams(this);
            if(_loc3_ != null)
            {
               _loc4_ = new CommandMessage();
               _loc4_.destination = _loc3_.destination;
               _loc4_.operation = CommandMessage.CLUSTER_REQUEST_OPERATION;
               _loc4_.messageRefType = _loc3_.messageRefType;
               _currentChannel.mx_internal::sendClusterRequest(new ClusterMessageResponder(_loc4_,this));
            }
            _hasRequestedClusterEndpoints = true;
         }
         if(_hunting)
         {
            param1.reconnecting = true;
            _hunting = false;
         }
         dispatchEvent(param1);
      }
      
      public function get messageAgents() : Array
      {
         return _messageAgents;
      }
      
      mx_internal function set channelFailoverURIs(param1:Object) : void
      {
         var _loc2_:int = 0;
         var _loc3_:int = 0;
         var _loc4_:Channel = null;
         _channelFailoverURIs = param1;
         _loc2_ = int(_channels.length);
         _loc3_ = 0;
         while(_loc3_ < _loc2_)
         {
            _loc4_ = _channels[_loc3_];
            if(_loc4_ == null)
            {
               break;
            }
            if(_channelFailoverURIs[_loc4_.id] != null)
            {
               _loc4_.failoverURIs = _channelFailoverURIs[_loc4_.id];
            }
            _loc3_++;
         }
      }
   }
}

import mx.core.mx_internal;
import mx.messaging.messages.IMessage;

use namespace mx_internal;

class PendingSend
{
   public var agent:MessageAgent;
   
   public var message:IMessage;
   
   public function PendingSend(param1:MessageAgent, param2:IMessage)
   {
      super();
      this.agent = param1;
      this.message = param2;
   }
}

class ClusterMessageResponder extends MessageResponder
{
   private var _channelSet:ChannelSet;
   
   public function ClusterMessageResponder(param1:IMessage, param2:ChannelSet)
   {
      super(null,param1);
      _channelSet = param2;
   }
   
   override protected function resultHandler(param1:IMessage) : void
   {
      var _loc2_:Object = null;
      var _loc3_:Array = null;
      var _loc4_:int = 0;
      var _loc5_:int = 0;
      var _loc6_:Object = null;
      var _loc7_:Object = null;
      if(param1.body != null && param1.body is Array)
      {
         _loc2_ = {};
         _loc3_ = param1.body as Array;
         _loc4_ = int(_loc3_.length);
         _loc5_ = 0;
         while(_loc5_ < _loc4_)
         {
            _loc6_ = _loc3_[_loc5_];
            for(_loc7_ in _loc6_)
            {
               if(_loc2_[_loc7_] == null)
               {
                  _loc2_[_loc7_] = [];
               }
               _loc2_[_loc7_].push(_loc6_[_loc7_]);
            }
            _loc5_++;
         }
         _channelSet.mx_internal::channelFailoverURIs = _loc2_;
      }
   }
}
