package mx.messaging
{
   import mx.core.mx_internal;
   import mx.logging.Log;
   import mx.messaging.messages.AcknowledgeMessage;
   import mx.messaging.messages.ErrorMessage;
   import mx.messaging.messages.IMessage;
   import mx.utils.Translator;
   
   use namespace mx_internal;
   
   public class Producer extends MessageAgent
   {
      private static const TRANSLATOR:Translator = Translator.getDefaultInstanceFor(Producer);
      
      private var _defaultHeaders:Object;
      
      private var _sendQueue:Array;
      
      public function Producer()
      {
         super();
         _log = Log.getLogger("mx.messaging.Producer");
         _agentType = "producer";
      }
      
      public function send(param1:IMessage) : void
      {
         var _loc2_:String = null;
         if(defaultHeaders != null)
         {
            for(_loc2_ in defaultHeaders)
            {
               if(!param1.headers.hasOwnProperty(_loc2_))
               {
                  param1.headers[_loc2_] = defaultHeaders[_loc2_];
               }
            }
         }
         if(Log.isInfo())
         {
            _log.info("\'{0}\' {1} sending message \'{2}\'",id,_agentType,param1.messageId);
         }
         if(_sendQueue == null)
         {
            internalSend(param1);
            if(clientId == null)
            {
               _sendQueue = [];
            }
         }
         else
         {
            _sendQueue.push(param1);
         }
      }
      
      public function get defaultHeaders() : Object
      {
         return _defaultHeaders;
      }
      
      public function set defaultHeaders(param1:Object) : void
      {
         _defaultHeaders = param1;
      }
      
      override public function acknowledge(param1:AcknowledgeMessage, param2:IMessage) : void
      {
         if(clientId == null && param1.clientId != null)
         {
            mx_internal::setClientId(param1.clientId);
         }
         super.acknowledge(param1,param2);
         flushSendQueue();
      }
      
      override public function fault(param1:ErrorMessage, param2:IMessage) : void
      {
         if(clientId == null && param1.clientId != null)
         {
            mx_internal::setClientId(param1.clientId);
         }
         super.fault(param1,param2);
         flushSendQueue();
      }
      
      private function flushSendQueue() : void
      {
         var _loc1_:int = 0;
         if(_sendQueue != null)
         {
            if(clientId != null || _sendQueue.length <= 1)
            {
               _loc1_ = 0;
               while(_loc1_ < _sendQueue.length)
               {
                  internalSend(_sendQueue[_loc1_] as IMessage);
                  _loc1_++;
               }
               _sendQueue = null;
            }
            else
            {
               internalSend(_sendQueue[0] as IMessage);
               _sendQueue = _sendQueue.splice(0,1);
            }
         }
      }
   }
}

